//
//  NSJSONSerialization+AGCategory.h
//  AGFoundation
//
//  Created by Andrew Garn on 14/03/2013.
//  Copyright (c) 2013 Andrew Garn. All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions are met:
//
//  1. Redistributions of source code must retain the above copyright notice, this
//  list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright notice,
//  this list of conditions and the following disclaimer in the documentation
//  and/or other materials provided with the distribution.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
//  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
//  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
//  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
//  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
//  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
//  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
//  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#import <Foundation/Foundation.h>

/** A collection of category extensions for `NSJSONSerialization` */
@interface NSJSONSerialization (AGCategory)

/** Parses the given JSON data in a background queue.
 @param data A data object containing JSON data.
 @param completionBlock The block object to call after successfully parsing the data.
 @param failureBlock The block object to call if an error occurs parsing the data. 
*/
+ (void)parseData_AG:(NSData *)data completionBlock:(void (^)(id JSONObject))completionBlock failureBlock:(void (^)(NSError *error))failureBlock;

/** Parses the given JSON data in a background queue.
 @param data A data object containing JSON data.
 @param options The options for reading the JSON data and creating the Foundation objects.
 @param completionBlock The block object to call after successfully parsing the data.
 @param failureBlock The block object to call if an error occurs parsing the data.
*/
+ (void)parseData_AG:(NSData *)data options:(NSJSONReadingOptions)options completionBlock:(void (^)(id JSONObject))completionBlock failureBlock:(void (^)(NSError *error))failureBlock;

@end